#!/usr/bin/env python

"""
Update the storage modules import manifest.

Copyright (C) 2016, 2017 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from glob import glob
from os import listdir
from os.path import commonprefix, isdir, join, split, splitext
import imiptools.stores

reserved = ["__init__.py", "common.py", "manifest.py"]

def get_extensions(dirname):
    filenames = []
    for filename in glob(join(dirname, "*.py")):
        leafname = split(filename)[-1]
        if leafname not in reserved:
            filenames.append(filename)
    return filenames

# The main program generating a new version of the manifest module.

if __name__ == "__main__":
    dirname = join(split(imiptools.stores.__file__)[0], "")
    manifest = join(dirname, "manifest.py")

    # Get all Python files in the stores directory, filtering out the
    # reserved files that do not provide storage functions.

    filenames = get_extensions(dirname)

    # Get all extensions from directories in the stores directory.

    for filename in listdir(dirname):
        filename = join(dirname, filename)
        if isdir(filename):
            filenames += get_extensions(filename)

    # Open the manifest module and write code to import and combine the
    # functions from each module.

    f = open(manifest, "w")
    try:
        print >>f, """\
stores = {}
"""

        for filename in filenames:
            relative = filename[len(commonprefix([filename, dirname])):]

            # NOTE: Converting POSIX paths to module paths.

            module = splitext(relative)[0].replace("/", ".")
            module_parts = module.rsplit(".", 1)

            # Get subpackage location and module.

            module_parents = len(module_parts) > 1 and module_parts[0]
            module_name = module_parts[-1]

            print >>f, """\
from imiptools.stores%s import %s
stores[%r] = %s
""" % (module_parents and ".%s" % module_parents or "",
       module_name, module_name, module_name)

    finally:
        f.close()

# vim: tabstop=4 expandtab shiftwidth=4
